<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services\Retorno;


use Fiscal\Entities\Boletpagar;
use Geral\Entities\Empresa;


/**
 * Class LeitorCnabCaixa
 * @package Financeiro\Logic\ContasPagar\Cnab\Retorno
 */
class LeitorCnabCaixa extends LeitorCnab{

    CONST ID_REGISTRO_HEADER_ARQUIVO = '0';
    CONST ID_REGISTRO_HEADER_LOTE  = '1';
    CONST ID_REGISTRO_DETALHE = '3';
    CONST ID_REGISTRO_TRAILLER_LOTE = '5';
    CONST ID_REGISTRO_TRAILLER_ARQUIVO = '9';
    CONST TIPO_INS_JUR_CPF = 1;
    CONST TIPO_INS_JUR_CNPJ = 2;
    CONST RETORNO = 2;
    CONST SEGMENTO_A = 'A';
    CONST SEGMENTO_B = 'B';
    CONST SEGMENTO_J = 'J';
    CONST SEGMENTO_K = 'K';
    CONST CODIGO_COMPENSACAO_CAIXA = '104';




    private $cdagencia;
    private $cdconta;
    private $ocorrenciaHeader;
    private $contHeaderLote = 0;
    private $cdfilial;
    private $nrlanbolpag;
    private $vencimento;


    /**
     * @param $arquivo
     * @param $cdempresa
     * @param $cdbanco
     * @throws \Exception
     */
    public function __construct($arquivo, $cdempresa, $cdbanco){
        try {

            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->cdempresa = $cdempresa;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function validaArquivo($linha, $estruturaArquivo){

        parent::setStatusArquivo(true);

        $idReg = substr($linha,7,1);
        if($this->contlinha == 1 and $idReg != self::ID_REGISTRO_HEADER_ARQUIVO ){
            self::gravaErros($this->contlinha, "Erro de identificação do registro header.",true);
            parent::setStatusArquivo(false);
        }
        if($this->contlinha > 1 and (! in_array($idReg, $estruturaArquivo)
        )){
            self::gravaErros($this->contlinha,"Identificação do registro não é válida",true);
            parent::setStatusArquivo(false);
        }

        return parent::getStatusArquivo();
    }

     public function getEstruturaArquivo(){
        return array(
                self::ID_REGISTRO_HEADER_ARQUIVO,
                self::ID_REGISTRO_HEADER_LOTE,
                self::ID_REGISTRO_DETALHE,
                self::ID_REGISTRO_TRAILLER_LOTE,
                self::ID_REGISTRO_TRAILLER_ARQUIVO
            );
    }

    public function leArquivo($nrorg){

        try {
            foreach ($this->arquivo as $linha) {
                $this->contlinha++;
                if($linha){

                    $this->retorno = new Retorno();

                    // Busca o Tipo de Registro do Arquivo
                    $idRegistro = substr($linha,7,1);

                    if(self::validaArquivo($linha, self::getEstruturaArquivo())){
                        switch ($idRegistro) {
                            case(self::ID_REGISTRO_HEADER_ARQUIVO):
                                self::leHeaderArquivo($linha, $nrorg);
                                break;

                            case(self::ID_REGISTRO_HEADER_LOTE):
                                self::leHeaderLote($linha);
                                break;

                            case(self::ID_REGISTRO_DETALHE):
                                self::leDetalhe($linha, $nrorg);
                                break;

                            case(self::ID_REGISTRO_TRAILLER_LOTE):
                                self::leTrailerLote($linha);
                                break;

                            case(self::ID_REGISTRO_TRAILLER_ARQUIVO):
                                break;
                        }
                    }
                    $this->retornos[] = $this->retorno;
                }
            }
            self::preparaOcorrencias($this->cdbanco, $nrorg);

        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * @param $cdempresa
     * @throws \Exception
     * @return
     */
    private function getInscricaoEmpresa($cdempresa, $nrorg){
        $empresa = Empresa::findOneByCdempresaAndNrorg($cdempresa, $nrorg);
        if($empresa)
            return $empresa->getNrinsjurempr();
        else
            throw new \Exception ('Empresa não Encontrada.');
    }


    /**
     * // =====================================================================  //
    // ====================  REGISTRO HEADER  DE ARQUIVO  ====================  //
    // =====================================================================  //
     * @param TXT $header
     */
    private function leHeaderArquivo($header, $nrorg){

        // CAMPO: Código do BCO na compensação
        // POSIÇÃO: 1 a 3
        // PICTURE: 9(3)
        if(substr($header,0,3) != self::CODIGO_COMPENSACAO_CAIXA){
            self::gravaErros($this->contlinha,'Código do banco inválido.', true);
            parent::setStatusArquivo(false);
        }

        // CAMPO: Tipo de inscrição da empresa
        // POSIÇÃO: 18 a 18
        // PICTURE: 9(1)
        $tipoInscricao = substr($header,17,1);
        if(empty($tipoInscricao)){
            self::gravaErros($this->contlinha,'Tipo de Inscrição da Empresa não Informado.', true);
            parent::setStatusArquivo(false);
        }


        $nrinsjurempr = self::getInscricaoEmpresa($this->cdempresa, $nrorg);
        // CAMPO: Número de inscrição da empresa
        // POSIÇÃO: 19 a 32
        // PICTURE: 9(14)
        if($tipoInscricao == self::TIPO_INS_JUR_CPF){
            $inscricao = substr($header,21,11);
        }else{
            $inscricao = substr($header,18,14);
        }
        if($nrinsjurempr <> $inscricao){
            self::gravaErros($this->contlinha,'Inscrição da empresa não cadastrada.', true);
            parent::setStatusArquivo(false);
        }

        // CAMPO: Código de Remessa/Retorno
        // POSIÇÃO: 143 a 143
        // PICTURE: 9(1)
        if(substr($header,142,1) != self::RETORNO){
            self::gravaErros($this->contlinha,'Código de Retorno Inválido.', true);
            parent::setStatusArquivo(false);
        }
    }

    /**
     * // =====================================================================  //
    // ====================  REGISTRO HEADER  DE LOTE  ====================  //
    // =====================================================================  //
     * @param TXT $header
     */
    private function leHeaderLote($header){
        // CAMPO: Número da agência centralizadora
        // POSIÇÃO: 53 a 57
        // PICTURE: 9(4)
        $this->cdagencia = substr($header,52,5);

        // CAMPO: Conta debitada com dígito
        // POSIÇÃO: 63 a 70
        // PICTURE: 9(8)
        $this->cdconta = substr($header,62,8);

        // CAMPO: Ocorrências
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $this->ocorrenciaHeader = substr($header,230,10);
        $this->contHeaderLote++;

    }

    /**
    // =====================================================================  //
    // ====================      REGISTRO DETALHE       ====================  //
    // =====================================================================  //
     * @param $detalhe
     */
    private function leDetalhe($detalhe, $nrorg){

        $segmento = substr($detalhe,13,1);

        $this->retorno->setIdheaderlote($this->contHeaderLote);

        switch($segmento){
            case self::SEGMENTO_A: $this->segmentoA($detalhe);
                break;
            case self::SEGMENTO_J: $this->segmentoJ($detalhe, $nrorg);
                break;
            case self::SEGMENTO_K: $this->segmentoK($detalhe);
                break;
        }
        
        if(($segmento == self::SEGMENTO_A) or ($segmento == self::SEGMENTO_J) or ($segmento == self::SEGMENTO_K)){
            
            $this->retorno->setCdlinha(str_pad($this->contlinha,10,'0',STR_PAD_LEFT));
            $this->retorno->setCdfilial($this->cdfilial);
            $this->retorno->setNrlanbolpag(str_pad($this->nrlanbolpag,10,'0',STR_PAD_LEFT));
            $this->retorno->setBoleto('BOLETO');
            $this->retorno->setCdconta($this->cdconta);
            $this->retorno->setCdagencia($this->cdagencia);
            $this->retorno->setDtvencimento($this->vencimento);
            //  $this->retorno->setCdtpbaixapag(self::getTipoBaixaPagamento($cdfilial, $nrlanbolpag, $nrorg));
            //  $this->retorno->setCdreducont(self::getReducont($cdfilial, $nrlanbolpag, $nrorg));
            // CAMPO: Tipo de movimento   [0 - Inclusão  9 - Exclusão]
            // POSIÇÃO: 15 a 15
            // PICTURE: 9(1)
            $tipomovimento = substr($detalhe,14,1);
            self::tipoMovimento($tipomovimento);
            // CAMPO: Ocorrência
            // POSIÇÃO: 231 a 240
            // PICTURE: X(10)
            $ocorrencia = substr($detalhe,230,10);
            $this->retorno->setOcorrencia($ocorrencia);
            if(!$ocorrencia or $ocorrencia == ' '){
                self::gravaErros($this->contlinha,'Ocorrência não informada.');
                parent::setStatusArquivo(false);
            }
            // =================  REGISTRO DO SEGMENTO J  =================  //
            /*if($segmento == self::SEGMENTO_J){

            }else{
                   // CAMPO: Data da efetivação do lançamento
                   // POSIÇÃO: 155 a 162
                   // PICTURE: 9(8)
                   $this->retorno->setDtpagamento(substr($detalhe,154,8));
                   // CAMPO: Valor do lançamento
                   // POSIÇÃO: 120 a 134
                   // PICTURE: 9(13)V9(2)
                   $varpago = floatval(substr($detalhe,119,15));
                   $this->retorno->setVrpago(round($varpago/100,2));
                   // CAMPO: Campo de uso exclusivo da empresa
                   // POSIÇÃO: 180 a 217
                   // PICTURE: X(20)
                   $nrret = substr($detalhe,179,38);
                   //if($nrret == false)
            }*/

        }
    }

    public function segmentoA($detalhe){
        $this->cdfilial = substr($detalhe,177,4);
        $this->nrlanbolpag = substr($detalhe,181,10);
        $this->vencimento = substr($detalhe,93,8);
        $this->retorno->setNossonumero(substr($detalhe,93,8));

        // CAMPO: Data da efetivação do lançamento
        // POSIÇÃO: 155 a 162
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($detalhe,154,8));

        // CAMPO: Valor do lançamento
        // POSIÇÃO: 120 a 134
        // PICTURE: 9(13)V9(2)
        $varpago = floatval(substr($detalhe,119,15));
        $this->retorno->setVrpago(round($varpago/100,2));

        // CAMPO: Campo de uso exclusivo da empresa
        // POSIÇÃO: 180 a 217
        // PICTURE: X(20)
        $nrret = substr($detalhe,179,38);
        //if($nrret == false)
    }

    public function segmentoJ($detalhe, $nrorg){
        $this->nrlanbolpag = substr($detalhe,182,10);
        $this->vencimento = substr($detalhe,91,8);
        $valordocumento = round(floatval(substr($detalhe,114,15))/100,2);
        $codBarras = substr($detalhe,17,44);
        //Busca a Filial do Boleto. Ocorre porque o layout não disponibiliza campo para a Filial
        $this->cdfilial = self::getFilialCaixa($this->nrlanbolpag, $codBarras, $valordocumento,
                                         substr($this->vencimento,0,2).'/'.substr($this->vencimento,2,2).'/'.
                                         substr($this->vencimento,4),
                                         $nrorg);



        /// CAMPO: Valor do deconto + Abatimento
        // POSIÇÃO: 115 a 129
        // PICTURE: 9(13)V9(2)
        $valordesconto = floatval(substr($detalhe,114,15));
        $this->retorno->setVrdesconto(round($valordesconto/100,2));
        // CAMPO: Valor da mora + Multa
        // POSIÇÃO: 130 a 144
        // PICTURE: 9(13)V9(2)
        $valoracrescimo = floatval(substr($detalhe,129,15));
        $this->retorno->setVracrescimo(round($valoracrescimo/100,2));
        // CAMPO: Data do pagamento
        // POSIÇÃO: 145 a 152
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($detalhe,144,8));
        // CAMPO: Valor do pagamento
        // POSIÇÃO: 153 a 167
        // PICTURE: 9(13)V9(2)
        $varpago = floatval(substr($detalhe,152,15));
        $this->retorno->setVrpago(round($varpago/100,2));
        // CAMPO: Valor do pagamento
        // POSIÇÃO: 100 a 114
        // PICTURE: 9(13)V9(2)
        $varpago = floatval(substr($detalhe,99,15));
        $this->retorno->setVrfinalpago(round($varpago/100,2));
        // CAMPO: Cód. Refer. atribuído pelo sacado
        // POSIÇÃO: 183 a 202
        // PICTURE: X(20)
        $nrret = substr($detalhe,182,38);
        if(empty($nrret))
            self::gravaErros($this->contlinha,'Número do Retorno não informado.');
    }

    public function segmentoK($detalhe){
        $this->cdfilial = substr($detalhe,73,6);
        $this->nrlanbolpag = substr($detalhe,73,6);
        $this->vencimento = substr($detalhe,36,8);
    }

    /**
     * @param $trailerlote
     */
    public function leTrailerLote($trailerlote){

        // CAMPO: Ocorrências
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $ocorrenciastrailer = trim(substr($trailerlote,230,10));

        /** @var $retorno Retorno  */
        foreach($this->retornos as &$retorno){

            if($retorno->getIdheaderlote() == $this->contHeaderLote){

                //Insere Situação do Cnab
                $retorno->setIdsitucnabp(self::situacaoCnab($retorno->getOcorrencia().
                    $ocorrenciastrailer.
                    $this->ocorrenciaHeader));

                //Adiciona as ocorrencias do trailler e do header.
                $retorno->setOcorrencia($retorno->getOcorrencia().
                    $ocorrenciastrailer.
                    $this->ocorrenciaHeader);
            }
        }
    }

    /**
     * @param $cdocorrencia
     * @return string
     */
    private function situacaoCnab($cdocorrencia){
        if ($cdocorrencia){
            if (strpos($cdocorrencia,'BD') !== false)
                return '20'; // Retorno Ok (Agendado)
            if (strpos($cdocorrencia,'02') !== false)
                return '00'; // Não Agendado
            if (strpos($cdocorrencia,'00') !== false)
                return '90'; // Baixado via CNAB
            if ((strpos($cdocorrencia,'BD') === false) and
                (strpos($cdocorrencia,'00') === false) and
                    (strpos($cdocorrencia,'02') === false))
                return '01'; // Retorno com erro
        }
        return '10'; //Sem retorno
    }


    /**
     * @param $nrlanbolpag
     * @param $cdbarrapag
     * @param $vrfinalbpag
     * @param $dtvenbolpag
     * @param $nrorg
     * @return mixed
     */
    private function getFilialCaixa($nrlanbolpag,$cdbarrapag, $vrfinalbpag, $dtvenbolpag, $nrorg){

        /** @var $boletpagar Boletpagar */
        $boletpagar =  Boletpagar::getObjectByWhere("LPAD(NRLANBOLPAG, 10, '0') = ? AND CDBARRAPAG = ?
                                                     AND VRFINALBPAG = ? AND DTVENBOLPAG = ? AND NRORG = ? ",
                                                     array($nrlanbolpag,$cdbarrapag, $vrfinalbpag, $dtvenbolpag,
                                                           $nrorg ));

        $filial = ($boletpagar) ? $boletpagar->getCdfilial() : null;
        return $filial;
    }

}