<?php

namespace Teknisa\FIN\Modules\ContaReceber\Controllers;

use Teknisa\Libs\Service\CRUD as CrudService;
use Teknisa\Libs\Util\Environment;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\FIN\Modules\ContaReceber\Services\RetornoCobrancaService;

/**
 * Class RetornoCobrancaController
 * @package Teknisa\FIN\Modules\ContaReceber\Controllers
 */
class RetornoCobrancaController extends TeknisaCrud{

    protected $dataSourceName = 'cnabrec_boletreceber_banco';
    protected $parameterBagColumns = array(
        "CDEMPRESA"     ,
        "DTCNABRECINI"  ,
        "DTCNABRECFIN"  ,
        "DTCNABRECTOT"  ,
        "NRNOTAFISCINI" ,
        "NRNOTAFISCFIN" ,
        "NRNOTAFISCTOT" ,
        "DTEMISSAOINI"  ,
        "DTEMISSAOFIN"  ,
        "DTEMISSAOTOT"  ,
        "DTVENBOLRECINI",
        "DTVENBOLRECFIN",
        "DTVENBOLRECTOT",
        "VRFINALBRECINI",
        "VRFINALBRECFIN",
        "VRFINALBRECTOT"
    );
    protected $services;

    /**
     * @param CrudService $crudService
     * @param ParameterBag $parameterBag
     * @param Environment $environment
     */
    public function __construct(CrudService $crudService, ParameterBag $parameterBag, Environment $environment, RetornoCobrancaService $services){
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->services     = $services;
    }

    /**
     * @param DTO\Request\Filter $request
     * @param DTO\Response $response
     */
    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $dataSource = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()));
        $this->dataSourceName = $dataSource;
        parent::find($request, $response);
    }

    /**
     * Carrega Retorno Cobrança
     * @param DTO\Request\Row $request
     * @param DTO\Response $response
     */
    public function carregaRetorno(DTO\Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow();

            $dataSet = $this->services->carregaRetorno($row);

            $response->addDataSet(new DataSet("BOLETOS", $dataSet[0]));
            $response->addDataSet(new DataSet("OCORRENCIAS", $dataSet[1]));
            $response->addDataSet(new DataSet("SITUACAOCNAB", $dataSet[2]));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    /**
     * Insere Retorno Cobrança
     * @param DTO\Request\DataSet $dataset
     * @param DTO\Response $response
     */
    public function insereRetorno(DTO\Request\DataSet $dataset, DTO\Response $response){
        try {
            // Timeout - Ilimitado
            set_time_limit(0);
            
            $rows = $dataset->getDataSet()->getRows();

            // Variaveis de Inicialização
            $cdempresa    = $rows[0]['CDEMPRESA'];
            $nomearquivo  = $rows[0]['NOMEARQUIVO'];
            $cdbanco      = $rows[0]['CDBANCO'];
            $boletos      = $rows[1];
            $ocorrencias  = $rows[2];

            //Insere Retorno Cobrança
            $this->services->insereRetorno($boletos, $ocorrencias, $nomearquivo, $cdempresa, $cdbanco);

        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    /**
     * Cancela Título Retorno
     * @param DTO\Request\DataSet $dataset
     * @param DTO\Response $response
     */
    public function cancelaTituloRetornoCob(DTO\Request\DataSet $dataset, DTO\Response $response){
        try {
            $rows = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $retorno = $rows[0];
            $boletos = $rows[1];

            // Cancela Título Retorno
            $this->services->cancelaTitulosRetornoCob($retorno, $boletos);

        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }
}