<?php
/**
 * Created by JetBrains PhpStorm.
 * User: leonardooliveira
 * Date: 30/05/14
 * Time: 11:36
 * To change this template use File | Settings | File Templates.
 */

namespace Financeiro\Logic\ContasReceber\Cnab\Remessa;
use Financeiro\Entities\Boletreceber;
use Financeiro\Entities\Emprbanc;
use Financeiro\Logic\ContasReceber\Cnab\Remessa\GeraCnab;
use Geral\Entities\Banco;
use Geral\Entities\Endeclie;
use Geral\Entities\Licencauso;
use Geral\Entities\Contcorr;
use Geral\Entities\Empresa;
use Geral\Util\DateUtil;
use Financeiro\Util\Util;
use Geral\Entities\Cliente;
class GeraCnabBradesco extends GeraCnab{

    const IDENTIFICACAO_REGISTRO = '0';
    const IDENTIFICACAO_ARQUIVO_REMESSA = '1';
    const LITERAL_REMESSA = 'REMESSA';
    const CODIGO_SERVICO = '01';
    const LITERAL_SERVICO = 'COBRANCA';
    const NUMERO_BRADESCO = '237';
    const NOME_BANCO_EXTENSO = 'BRADESCO';
    const IDENTIFICACAO_SISTEMA = 'MX';
    const IDENTIFICACAO_DESCONTO = 'DESC';
    const NUMERO_SEQUENCIAL_ARQUIVO = '1';
    const IDENTIFICACAO_REGISTRO_TRAILLER = '9';
    const IDENTIFICACAO_REGISTRO_TRANSACAO = '1';

    public function geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnab, $cdreducont, $carteira = NULL){
        try {
            if (!is_dir($caminho))
                mkdir($caminho, "0777", true);
            $nmarquivo = $caminho.$nmarquivo;
            $file = fopen($nmarquivo, "w+");

            tk_set_env_var('REGISTROS', tk_get_env_var('REGISTROS') + 1);

            //#################################################################//
            //############### REGISTRO HEADER DE ARQUIVO ######################//
            //#################################################################//

            // CAMPO: Identificação do Registro
            // POSIÇÃO: 001 a 001
            // PICTURE: 9(1)
            $arquivo = self::IDENTIFICACAO_REGISTRO;

            // CAMPO: Identificação do Arquivo de Remessa
            // POSIÇÃO: 002 a 002
            // PICTURE: 9(1)
            $arquivo .= self::IDENTIFICACAO_ARQUIVO_REMESSA;

            // CAMPO: Literal Remessa
            // POSIÇÃO: 003 a 009
            // PICTURE: X(7)
            $arquivo .= self::LITERAL_REMESSA;

            // CAMPO: Código de Serviço
            // POSIÇÃO: 010 a 011
            // PICTURE: 9(2)
            $arquivo .= self::CODIGO_SERVICO;

            // CAMPO: Literal Serviço
            // POSIÇÃO: 012 a 026
            // PICTURE: X(15)
            $arquivo .= self::LITERAL_SERVICO;

            $arquivo = str_pad($arquivo, 26);

            // CAMPO: Código da Empresa
            // POSIÇÃO: 027 a 046
            // PICTURE: 9(20)
            $codigoEmpresa = self::getCodigoEmpresa($cdbanco, $cdempresa, $cdidrec);
            if ($codigoEmpresa)
                $arquivo .= $codigoEmpresa;
            else
                $arquivo = str_pad($arquivo, 46, "0");

            // CAMPO: Nome da Empresa
            // POSIÇÃO: 047 a 076
            // PICTURE: X(30)
            /** @var $empresa Empresa */
            $empresa = Empresa::findOneByCdempresa($cdempresa);
            if ($empresa) {
                $nmrazsocemp = Util::campoCnabValido($empresa->getNmrazsocempr());
                $nmrazsocemp = substr($nmrazsocemp, 0, 30);
                $arquivo .= str_pad($nmrazsocemp, 30);
            }
            else
                $arquivo = str_pad($arquivo, 76);

            // CAMPO: Número do Bradesco na Câmara de Compensação
            // POSIÇÃO: 077 a 079
            // PICTURE: 9(3)
            $arquivo .= self::NUMERO_BRADESCO;

            // CAMPO: Nome do Banco por Extenso
            // POSIÇÃO: 080 a 094
            // PICTURE: X(15)
            $arquivo .= str_pad(self::NOME_BANCO_EXTENSO, 15);

            // CAMPO: Data da Gravação do Arquivo
            // POSIÇÃO: 095 a 100
            // PICTURE: 9(6)
            $now = DateUtil::getDataDeString(DateUtil::truncateString(now()));
            $data = DateUtil::getDia($now) . DateUtil::getMes($now) . substr(DateUtil::getAno($now), -2);
            $arquivo .= Util::campoCnabValido($data);

            // CAMPO: Branco
            // POSIÇÃO: 101 a 108
            // PICTURE: X(8)
            $arquivo = str_pad($arquivo, 108);

            // CAMPO: Identificação do Sistema
            // POSIÇÃO: 109 a 110
            // PICTURE: X(2)
            $arquivo .= self::IDENTIFICACAO_SISTEMA;

            // CAMPO: Nº Seqüencial do Arquivo
            // POSIÇÃO: 111 a 117
            // PICTURE: 9(7)
            $arquivo .= substr($nrcnab, -7);

            if (tk_get_context_vars('IDTIPCNABREC', 'wzd_stage_pbi00109_001_win_pbi00109_001') == '0')
                // CAMPO: Branco
                // POSIÇÃO: 118 a 394
                // PICTURE: X(277)
                $arquivo = str_pad($arquivo, 394);
            else {
                // CAMPO: Branco
                // POSIÇÃO: 118 a 366
                // PICTURE: X(249)
                $arquivo = str_pad($arquivo, 366);

                // CAMPO: Identificação de Desconto
                // POSIÇÃO: 367 a 370
                // PICTURE: X(4)
                $arquivo .= self::IDENTIFICACAO_DESCONTO;
                /** @var $contcorr Contcorr */
                $contcorr = Contcorr::findOneByCdreducont($cdreducont);

                // CAMPO: Número de Autorização
                // POSIÇÃO: 371 a 374
                // PICTURE: X(4)
                //frmRemessaCNAB.cdsContaCorr.Close;
                //frmRemessaCNAB.cdsContaCorr.Open;
                if(!$contcorr->getNrautcnabdes())
                    throw new \Exception('Operação bloqueada! Número de Autorização para geração de CNAB Desconto não cadastrado.');
                else
                    $arquivo .= $contcorr->getNrautcnabdes();

                // CAMPO: Código da Agência, sem dígito
                // POSIÇÃO: 375 a 379
                // PICTURE: X(5)
                $arquivo .= Util::campoCnabValido(str_pad(substr($contcorr->getCdagencia(), 0, 4), 5, '0', STR_PAD_LEFT));

                // CAMPO: Conta Corrente + Dígito da Conta Corrente
                // POSIÇÃO: 380 a 387
                // PICTURE: X(8)
                $arquivo .= Util::campoCnabValido(str_pad($contcorr->getCdcontcorr(), 8, '0', STR_PAD_LEFT));

                // CAMPO: Branco
                // POSIÇÃO: 388 a 394
                // PICTURE: X(7)
                $arquivo = str_pad($arquivo, 394);
            }

            // CAMPO: Nº Seqüencial do Registro de Um em Um
            // POSIÇÃO: 395 a 400
            // PICTURE: 9(6)
            $arquivo .= str_pad(tk_get_env_var('REGISTROS'), 6, '0', STR_PAD_LEFT) . "\r\n";

            //Grava a variavel arquivo no arquivo texto
            fprintf($file, "%s", $arquivo);
            fclose($file);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraTransacao(
        $caminho, $nmarquivo, $cdempresa, $cdcartcobbol, $cdagencia, $cdcontcorr, $cdfilial,
        $nrlanbolrec, $vrpemobrec, $idimprboleto, $cdnosnumrec, $dtvenbolrec, $cdins1cobbol,
        $qtdiaprotbol, $vrmordiabrec, $vrfinalbrec, $idopecnabrec, $vrpemulbrec, $idtpijurclie,
        $nrinsjurclie, $cdcliente, $cdins2cobbol
    ){
        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        tk_set_env_var('REGISTROS', tk_get_env_var('REGISTROS') + 1);

        // CAMPO: Identificação do Registro
        // POSIÇÃO: 001 a 001
        // PICTURE: 9(1)
        $arquivo = self::IDENTIFICACAO_REGISTRO_TRANSACAO;

        if (tk_get_context_vars('IDTIPCNABREC', 'wzd_stage_pbi00109_001_win_pbi00109_001') == '0'){

            // CAMPO: Agência do Débito
            // POSIÇÃO: 002 a 006
            // PICTURE: 9(5)
            $arquivo = str_pad($arquivo, 6, '0');

            // CAMPO: Dígito da Agência de Débito
            // POSIÇÃO: 007 a 007
            // PICTURE: X(1)
            $arquivo = str_pad($arquivo, 7);

            // CAMPO: Razão da Conta Corrente
            // POSIÇÃO: 008 a 012
            // PICTURE: 9(5)
            $arquivo = str_pad($arquivo, 12, '0');

            // CAMPO: Conta Corrente
            // POSIÇÃO: 013 a 019
            // PICTURE: 9(7)
            $arquivo = str_pad($arquivo, 19, '0');

            // CAMPO: Dígito da conta corrente
            // POSIÇÃO: 020 a 020
            // PICTURE: X(1)
            $arquivo = str_pad($arquivo, 20);

            // CAMPO: Identificação da Empresa Cedente no Banco
            // POSIÇÃO: 021 a 037
            // PICTURE: X(17)
            $arquivo .= '0' . str_pad($cdcartcobbol, 3, '0', STR_PAD_LEFT) . str_pad(substr($cdagencia, 0, 4), 5, '0', STR_PAD_LEFT) . str_pad($cdcontcorr, 8, '0', STR_PAD_LEFT);
        }
        else {
            // CAMPO: Tipo de Inscrição
            // POSIÇÃO: 002 a 003
            // PICTURE: 9(2)
            /** @var $empresa Empresa */
            $empresa = Empresa::findOneByCdempresa($cdempresa);
            if($empresa){
                switch ($empresa->getIdtpijurempr()){
                    case 'F':
                        $arquivo .= '01';
                        break;
                    case 'J':
                        $arquivo .= '02';
                        break;
                    case 'I':
                    case 'O':
                        $arquivo .= '98';
                        break;
                }
            }
            else {
                $arquivo .= '99';
            }

            // CAMPO: Número de Inscrição
            // POSIÇÃO: 004 a 017
            // PICTURE: 9(14)
            if($empresa)
                $arquivo .= substr($empresa->getNrinsjurempr(), 0, 14);
            else
                $arquivo = str_pad($arquivo, 17, '0');

            // CAMPO: Identificação da Empresa
            // POSIÇÃO: 018 a 037
            // PICTURE: 9(20)
            $arquivo .= '0000' . str_pad($cdcartcobbol, 3, '0', STR_PAD_LEFT) . str_pad(substr($cdagencia, 0, 4), 5, '0', STR_PAD_LEFT) . str_pad($cdcontcorr, 8, '0', STR_PAD_LEFT);
        }

        // CAMPO: Nº do Controle do Participante
        // POSIÇÃO: 038 a 062
        // PICTURE: X(25)
        $arquivo .= str_pad($cdfilial . $nrlanbolrec, 25);

        if (tk_get_context_vars('IDTIPCNABREC', 'wzd_stage_pbi00109_001_win_pbi00109_001') == '0'){
            // CAMPO: Código do Banco a ser debitado na Câmara de Compensação
            // POSIÇÃO: 063 a 065
            // PICTURE: 9(3)
            $arquivo .= '000'; // Usado apenas para débito automático

            if (!$vrpemobrec) {
                // CAMPO: Campo de Multa
                // POSIÇÃO: 066 a 066
                // PICTURE: 9(1)
                $arquivo .= '0';

                // CAMPO: Campo de Multa
                // POSIÇÃO: 067 a 070
                // PICTURE: 9(1)
                $arquivo .= '0000';
            }
            else {
                // CAMPO: Campo de Multa
                // POSIÇÃO: 066 a 066
                // PICTURE: 9(1)
                $arquivo .= '2';

                // CAMPO: Campo de Multa
                // POSIÇÃO: 067 a 070
                // PICTURE: 9(1)
                $vr = str_pad($vrpemulbrec * 100, 4, '0', STR_PAD_LEFT);
                $arquivo .= Util::campoCnabValido($vr);
           }
        }
        else {
            // CAMPO: Zeros
            // POSIÇÃO: 063 a 070
            // PICTURE: 9(8)
            $arquivo = str_pad($arquivo, 70, '0');
        }

        // CAMPO: Identificação do título no banco
        // POSIÇÃO: 071 a 082
        // PICTURE: X(12)
        $idImprBol = 'E';
        if ($idimprboleto){
            $idImprBol = $idimprboleto;
        }
        if ($idImprBol == 'E') {
            if ($cdnosnumrec) {
                $arquivo .= str_pad($cdnosnumrec, 12, '0', STR_PAD_LEFT);
            }
            else {
                $arquivo = str_pad($arquivo, 82, '0');
            }
        }
        else {
            $arquivo = str_pad($arquivo, 82, '0');
        }

        if (tk_get_context_vars('IDTIPCNABREC', 'wzd_stage_pbi00109_001_win_pbi00109_001') == '0'){

            // CAMPO: Desconto Bonificação por dia
            // POSIÇÃO: 083 a 092
            // PICTURE: 9(10)
            $arquivo = str_pad($arquivo, 92, '0');

            // CAMPO: Condição para Emissão da Papelete de Cobrança
            // POSIÇÃO: 093 a 093
            // PICTURE: 9(1)
            if ($idimprboleto == 'B')
                $arquivo .= '1';
            else
                $arquivo .= '2';

            // CAMPO: Ident. se emite papeleta para débito automático
            // POSIÇÃO: 094 a 094
            // PICTURE: X(1)
            $arquivo .= ' ';

            // CAMPO: Identificação da Operação do banco
            // POSIÇÃO: 095 a 104
            // PICTURE: X(10)
            $arquivo = str_pad($arquivo, 104, ' ');

            // CAMPO: Indicador Rateio Crédito
            // POSIÇÃO: 105 a 105
            // PICTURE: X(1)
            $arquivo .= ' ';

            // CAMPO: Endereçamento para Aviso do Débito Automático em Conta Corrente
            // POSIÇÃO: 106 a 106
            // PICTURE: 9(1)
            $arquivo .= '0';

            // CAMPO: Branco
            // POSIÇÃO: 107 a 108
            // PICTURE: X(2)
            $arquivo .= '  ';
        }
        else {
            // CAMPO: Zeros
            // POSIÇÃO: 083 a 092
            // PICTURE: 9(10)
            $arquivo = str_pad($arquivo, 92, '0');

            // CAMPO: Ident. Operação no Bradesco
            // POSIÇÃO: 093 a 108
            // PICTURE: 9(16)
            $arquivo = str_pad($arquivo, 108, ' ');
        }

        // CAMPO: Identificação ocorrência
        // POSIÇÃO: 109 a 110
        // PICTURE: 9(2)
        // CONTEÚDO: 01 - Remessa                                  02 - Pedido de baixa
        //           04 - Concessão de abatimento                  05 - Cancelamento de abatimento concedido
        //           06 - Alteração de vencimento                  07 - Alteração do controle do participante
        //           08 - Alteração de seu número                  09 - Pedido de protesto
        //           18 - Sustar protesto e baixar Título          19 - Sustar protesto e manter em carteira
        //           31 - Alteração de outros dados                35 - Desagendamento do débito automático
        //           68 - Acerto nos dados do rateio de Crédito    69 - Cancelamento do rateio de crédito
        if($idopecnabrec)
            $arquivo .= '01';
        else if($idopecnabrec)
            $arquivo .= '02';
        else
            $arquivo .= '06';

        // CAMPO: Nº do documento
        // POSIÇÃO: 111 a 120
        // PICTURE: X(10)
        /** @var $entitieDup \Financeiro\Repositories\Boletreceber */
        $entitieDup = Boletreceber::getRepository();
        $duplicata = $entitieDup->getDuplicata($cdfilial, $nrlanbolrec);
        if($duplicata)
            $arquivo .= Util::campoCnabValido(substr($duplicata[0]['DUPLICATA'], 0, 8) . $duplicata[0]['NRPARCELA']);
        else
            $arquivo .= $nrlanbolrec;

        // CAMPO: Data do Vencimento do Título
        // POSIÇÃO: 121 a 126
        // PICTURE: 9(6)
        $vencimento = DateUtil::getDataDeString(DateUtil::truncateString($dtvenbolrec));
        $vencimento = DateUtil::getDia($vencimento) . DateUtil::getMes($vencimento) . substr(DateUtil::getAno($vencimento), -2);
        $arquivo .= Util::campoCnabValido($vencimento);

        // CAMPO: Valor do título
        // POSIÇÃO: 127 a 139
        // PICTURE: 9(13)
        $vr = $vrfinalbrec * 100;
        $vr = str_pad($vr, 13, "0", STR_PAD_LEFT);
        $arquivo .= Util::campoCnabValido($vr);

        if (tk_get_context_vars('IDTIPCNABREC', 'wzd_stage_pbi00109_001_win_pbi00109_001') == '0'){
           // CAMPO: Banco encarregado da cobrança
           // POSIÇÃO: 140 a 142
           // PICTURE: 9(3)
            $arquivo .= '000';

           // CAMPO: Agência do depositário
           // POSIÇÃO: 143 a 147
           // PICTURE: 9(5)
            $arquivo .= '00000';
        }
        else {
           // CAMPO: Branco
           // POSIÇÃO: 140 a 147
           // PICTURE: 9(8)
            $arquivo = str_pad($arquivo, 147, ' ');
        }

        // CAMPO: Espécie do título
        // POSIÇÃO: 148 a 149
        // PICTURE: 9(2)
        $arquivo .= '01';

        if (tk_get_context_vars('IDTIPCNABREC', 'wzd_stage_pbi00109_001_win_pbi00109_001') == '0'){

            // CAMPO: Identificação
            // POSIÇÃO: 150 a 150
            // PICTURE: X(1)
            $arquivo .= 'N';

            // CAMPO: Data da emissão do título
            // POSIÇÃO: 151 a 156
            // PICTURE: 9(6)
            $now = DateUtil::getDataDeString(DateUtil::truncateString(now()));
            $data = DateUtil::getDia($now) . DateUtil::getMes($now) . substr(DateUtil::getAno($now), -2);
            $arquivo .= Util::campoCnabValido($data);

            // CAMPO: 1ª instrução
            // POSIÇÃO: 157 a 158
            // PICTURE: 9(2)
            if($cdins1cobbol)
                $arquivo .= $cdins1cobbol;
            else
                $arquivo .= '00';

            // CAMPO: 2ª instrução
            // POSIÇÃO: 159 a 160
            // PICTURE: 9(2)
            if($qtdiaprotbol)
                $arquivo .= $qtdiaprotbol;
            else
                $arquivo .= '00';

            // CAMPO: Valor a ser cobrado por dia de atraso
            // POSIÇÃO: 161 a 173
            // PICTURE: 9(13)

            //DÚVIDA: a ordem disso tá certa?
            if(!$vrmordiabrec){
                if(!$vrpemobrec){
                    $arquivo = str_pad($arquivo, 173, '0');
                }
                else{
                    $arquivo .= str_pad(str_replace('.', '', ($vrfinalbrec * $vrpemobrec) / 100), 13, '0', STR_PAD_LEFT);
                }
            }
            else{
                $arquivo .= str_pad(str_replace('.', '', $vrmordiabrec * 100), 13, '0', STR_PAD_LEFT);
            }
            // CAMPO: Data limite para concessão de desconto
            // POSIÇÃO: 174 a 179
            // PICTURE: 9(6)

            //DÚVIDA: ZEROS??
            $arquivo = str_pad($arquivo, 179, '0');

            // CAMPO: Valor do desconto
            // POSIÇÃO: 180 a 192
            // PICTURE: 9(13)
            $arquivo = str_pad($arquivo, 192, '0');

            // CAMPO: Valor do IOF
            // POSIÇÃO: 193 a 205
            // PICTURE: 9(13)
            $arquivo = str_pad($arquivo, 205, '0');

           // CAMPO: Valor do Abatimento a ser concedido ou cancelado
           // POSIÇÃO: 206 a 218
           // PICTURE: 9(13)
           $arquivo = str_pad($arquivo, 218, '0');
        }
        else {
           // CAMPO: Branco
           // POSIÇÃO: 150 a 150
           // PICTURE: X(1)
            $arquivo .= ' ';

           // CAMPO: Zeros
           // POSIÇÃO: 151 a 218
           // PICTURE: X(68)
            $arquivo = str_pad($arquivo, 218, '0');
        }

        // ###############################################
        // VERIFICA LICENÇA DE USO DO CLIENTE  ###########
        // ###############################################
        /** @var $entitie \Geral\Repositories\Licencauso */
        $entitie = Licencauso::getRepository();
        $licencaUso = $entitie->getLicencaUso($cdfilial, $nrlanbolrec);


        if ($licencaUso){
            // CAMPO: Identificação do tipo de inscrição do Sacado
            // POSIÇÃO: 219 a 220
            // PICTURE: 9(2)
            if (!$licencaUso[0]['IDTPIJURLUSO']){
                switch ($licencaUso[0]['IDTPIJURLUSO']){
                    case 'F':
                        $arquivo .= '01';
                        break;
                    case 'J':
                        $arquivo .= '02';
                        break;
                    case 'I':
                    case 'O':
                        $arquivo .= '99';
                }
            }
            else{
                $arquivo .= '98';
            }
            // CAMPO: Nº Inscrição do Sacado
            // POSIÇÃO: 221 a 234
            // PICTURE: 9(14)
            if (!$licencaUso[0]['NRINSJURLUSO']){
                $arquivo = str_pad($arquivo, 234, '0');
            }
            else
                $arquivo .= str_pad($licencaUso[0]['NRINSJURLUSO'], 14, '0', STR_PAD_LEFT);

            // CAMPO: Nome do Sacado
            // POSIÇÃO: 235 a 274
            // PICTURE: X(40)
            $razaoSocial = Util::campoCnabValido($licencaUso[0]['NMRAZSOCLUSO']);
            $razaoSocial = substr($razaoSocial, 0, 40);
            $arquivo .= str_pad($razaoSocial, 40);

            // CAMPO: Endereço do Sacado
            // POSIÇÃO: 275 a 314
            // PICTURE: X(40)
            $endeUso = Util::campoCnabValido($licencaUso[0]['DSENDELUSO']);
            $endeUso = substr($endeUso, 0, 40);
            $arquivo .= str_pad($endeUso, 40);
        }
        else {
            // CAMPO: Identificação do tipo de inscrição do Sacado
            // POSIÇÃO: 219 a 220
            // PICTURE: 9(2)
            if ($idtpijurclie) {
                switch ($idtpijurclie){
                    case 'F':
                        $arquivo .= '01';
                        break;
                    case 'J':
                        $arquivo .= '02';
                        break;
                    case 'I':
                    case 'O':
                        $arquivo .= '99';
                }
            }
            else
                $arquivo .= '98';

            // CAMPO: Nº Inscrição do Sacado
            // POSIÇÃO: 221 a 234
            // PICTURE: 9(14)
            if(!$nrinsjurclie)
                $arquivo = str_pad($arquivo, 234, '0');
            else
                $arquivo .= str_pad($nrinsjurclie, 14, '0', STR_PAD_LEFT);

            // CAMPO: Nome do Sacado
            // POSIÇÃO: 235 a 274
            // PICTURE: X(40)
            /** @var $cliente Cliente */
            $cliente = Cliente::findOneByCdcliente($cdcliente);
            $razaoSocial = Util::campoCnabValido($cliente->getNmrazsocclie());
            $razaoSocial = substr($razaoSocial, 0, 40);
            $arquivo .= str_pad($razaoSocial, 40);

            //*****************************************************//
            //*************  Endereço do Cliente  *****************//
            /** @var $entitieEndeClie \Geral\Repositories\Endeclie */
            $entitieEndeClie = Endeclie::getRepository();
            $endeclie = $entitieEndeClie->getEndeClie($cdcliente);

            // CAMPO: Endereço do Sacado
            // POSIÇÃO: 275 a 314
            // PICTURE: X(40)
            $enderecoCliente = Util::campoCnabValido($endeclie[0]['DSENDECLIE']);
            $enderecoCliente = substr($enderecoCliente, 0, 40);
            $arquivo .= str_pad($enderecoCliente, 40);
        }

        if (tk_get_context_vars('IDTIPCNABREC', 'wzd_stage_pbi00109_001_win_pbi00109_001') == '0'){
           // CAMPO: 1ª Mensagem
           // POSIÇÃO: 315 a 326
           // PICTURE: X(12)
            $arquivo = str_pad($arquivo, 326, ' ');
        }
        else {
           // CAMPO: Branco
           // POSIÇÃO: 315 a 326
           // PICTURE: X(12)
            $arquivo = str_pad($arquivo, 326, ' ');
        }

        // CAMPO: CEP + Sufixo do CEP
        // POSIÇÃO: 327 a 334
        // PICTURE: 9(5)
        if ($licencaUso){
           if (!$licencaUso[0]['NRCEPLUSO'])
              $arquivo = str_pad($arquivo, 334, '0');
           else
              $arquivo .= Util::campoCepTel($licencaUso[0]['NRCEPLUSO']);
        }
        else {
            if (!$endeclie[0]['NRCEPCLIE'])
                $arquivo = str_pad($arquivo, 334, '0');
            else
                $arquivo .= Util::campoCepTel($endeclie[0]['NRCEPCLIE']);
        }

        if (tk_get_context_vars('IDTIPCNABREC', 'wzd_stage_pbi00109_001_win_pbi00109_001') == '0'){
            // CAMPO: Sacador/Avalista ou 2ª Mensagem
            // POSIÇÃO: 335 a 394
            // PICTURE: X(60)
            /** @var $entitieBoleto \Financeiro\Repositories\Boletreceber */
            $entitieBoleto = Boletreceber::getRepository();
            $txboletos = $entitieBoleto->getTxBoleto($cdfilial, $nrlanbolrec);
            $mensagem = Util::campoCnabValido($txboletos[0]['TXMENSBOLBOL']);
            $mensagem = substr($mensagem, 0, 60);
            $arquivo .= str_pad($mensagem, 60);
        }
        else {
           // CAMPO: Branco
           // POSIÇÃO: 335 a 394
           // PICTURE: X(60)
            $arquivo = str_pad($arquivo, 394, ' ');
        }

        // CAMPO: Nº Seqüencial do Registro
        // POSIÇÃO: 395 a 400
        // PICTURE: 9(6)
        $arquivo .= str_pad(tk_get_env_var('REGISTROS'), 6, '0', STR_PAD_LEFT) . "\r\n";

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    public function GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco){
        if (!is_dir($caminho))
            mkdir($caminho, "0777", true);
        $nmarquivo = $caminho.$nmarquivo;
        $file = fopen($nmarquivo, "a+");

        tk_set_env_var('REGISTROS', tk_get_env_var('REGISTROS') + 1);

        // CAMPO: Identificação do registro
        // POSIÇÃO: 001 a 001
        // PICTURE: 9(1)
        $arquivo = self::IDENTIFICACAO_REGISTRO_TRAILLER;

        // CAMPO: Branco
        // POSIÇÃO: 002 a 394
        // PICTURE: X(393)
        $arquivo = str_pad($arquivo, '394');

        // CAMPO: Nº Seqüencial do Registro
        // POSIÇÃO: 395 a 400
        // PICTURE: 9(6)
        $arquivo .= str_pad(tk_get_env_var('REGISTROS'), 6, '0', STR_PAD_LEFT);

        //Grava a variavel arquivo no arquivo texto
        fprintf($file, "%s", $arquivo);
        fclose($file);
    }

    private static function getCodigoEmpresa($cdbanco, $cdempresa, $cdidrec){
        $resultado = '';
        /** @var $banco Banco */
        $banco = Banco::findOneByCdbanco($cdbanco);
        $banccnab = $banco->getCdbanccnab();

        if($banccnab == '001'){
            /** @var $emprbanc Emprbanc */
            $emprbanc = Emprbanc::findOneByCdbancoAndCdempresa($cdbanco, $cdempresa);
        }

        if($cdidrec)
            $resultado = $cdidrec;

        else{
            if($banccnab != '001')
                /** @var $emprbanc Emprbanc */
                $emprbanc = Emprbanc::findOneByCdbancoAndCdempresa($cdbanco, $cdempresa);

            if($emprbanc->getCdempbancrec()){
                $resultado = $emprbanc->getCdempbancrec();
            }
        }
        return $resultado;
    }

    public function geraHeaderLote($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec, $nrcnabrec, $cdreducont, $carteira = NULL){}
    public function geraDetalheSegmentoP($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrseqremessarec,
        $nrcnabrec, $idoperacao, $cdagenciacnab, $cdcontacorrente, $cdcontcorrcobranca, $nossonumero,
        $idtipocobranca, $nrlanbolrec, $cdfilial, $dtvenbolrec, $valorfinalboleto, $valorpemobrec, $valormordiabrec,
        $valorpemulbrec, $valormultabrec, $valorfinalbrec, $cdcartcobbol, $cddiaprotbol, $cdreducont, $imprbol,
        $qtdiaprotbol){}
    public function geraDetalheSegmentoQ($caminho, $nmarquivo, $nrorg, $cdbanco, $cdcliente, $idoperacao, $cdempresa, $cdfilial, $nrlanbolrec){}
    public function geraDetalheSegmentoR($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $dtvenbolrec, $valorpemulbrec, $nrlanbolrec, $valormultabrec){}
    public function GeraTraillerLote($caminho, $nmarquivo, $nrorg, $cdbanco){}
}